package com.buystars.ScreenRecording;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.hardware.display.DisplayManager;
import android.hardware.display.VirtualDisplay;
import android.media.MediaCodec;
import android.media.MediaCodecInfo;
import android.media.MediaFormat;
import android.media.projection.MediaProjection;
import android.media.projection.MediaProjectionManager;
import android.os.IBinder;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Surface;

import net.ossrs.rtmp.ConnectCheckerRtmp;
import net.ossrs.rtmp.SrsFlvMuxer;

import java.io.IOException;
import java.nio.ByteBuffer;

import android.os.Handler;

import androidx.core.app.NotificationCompat;

import com.buystars.R;


public class ScreenRecordingService extends Service implements ConnectCheckerRtmp {
    private static final String TAG = "ScreenRecordingService";
    private static final int WIDTH = 720;
    private static final int HEIGHT = 1280;
    private static final int DENSITY = DisplayMetrics.DENSITY_HIGH;
    private static final int BITRATE = 1024 * 1024; // 1 Mbps
    private static final int FRAME_RATE = 30;
    private static final int NOTIFICATION_ID = 1190;
    private NotificationManager notificationManager;
    private NotificationCompat.Builder notificationBuilder;

    private MediaProjectionManager projectionManager;
    private MediaProjection mediaProjection;
    private VirtualDisplay virtualDisplay;
    private MediaCodec mediaCodec;
    private Surface inputSurface;

    private SrsFlvMuxer srsFlvMuxer;

    private Handler handler;

    @Override
    public void onCreate() {
        super.onCreate();
        projectionManager = (MediaProjectionManager) getSystemService(Context.MEDIA_PROJECTION_SERVICE);
        srsFlvMuxer = new SrsFlvMuxer(this);
        notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        notificationBuilder = new NotificationCompat.Builder(this, getString(R.string.screen_recording_channel))
                .setContentTitle(getString(R.string.app_name))
                .setContentText(getString(R.string.screen_recording_text))
                .setSmallIcon(android.R.drawable.stat_sys_download)
                .setPriority(NotificationCompat.PRIORITY_LOW);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (intent != null && intent.getAction() != null && intent.getAction().equals("START")) {
            handler = new Handler();
            startForeground(NOTIFICATION_ID, notificationBuilder.build());
            notificationManager.notify(NOTIFICATION_ID, notificationBuilder.build());
            handler.postDelayed(()->{
                    startScreenRecording(intent);
            }, 1000);
        } else if (intent != null && intent.getAction() != null && intent.getAction().equals("STOP")) {
            stopScreenRecording();
        }
        return START_STICKY;
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    private MediaProjection.Callback mediaProjectionCallback = new MediaProjection.Callback() {
        @Override
        public void onStop() {
            super.onStop();
            // Handle the stop event, release resources
            stopScreenRecording();
            Log.d(TAG, "MediaProjection stopped");
        }
    };
    private void startScreenRecording(Intent intent) {
        int resultCode = intent.getIntExtra("resultCode", -1);
        Intent data = intent.getParcelableExtra("data");
        assert data != null;
        mediaProjection = projectionManager.getMediaProjection(resultCode, data);
        mediaProjection.registerCallback(mediaProjectionCallback, handler);
        String rtmpUrl = "rtmp://a.rtmp.youtube.com/live2/ssh6-0wyb-8u48-b7z2-dbfy";
        srsFlvMuxer.start(rtmpUrl);
        setupMediaCodec();
        setupVirtualDisplay();
    }

    private void setupMediaCodec() {
        try {
            mediaCodec = MediaCodec.createEncoderByType("video/avc");
            MediaFormat format = MediaFormat.createVideoFormat("video/avc", WIDTH, HEIGHT);
            format.setInteger(MediaFormat.KEY_BIT_RATE, BITRATE);
            format.setInteger(MediaFormat.KEY_FRAME_RATE, FRAME_RATE);
            format.setInteger(MediaFormat.KEY_COLOR_FORMAT, MediaCodecInfo.CodecCapabilities.COLOR_FormatSurface);
            format.setInteger(MediaFormat.KEY_I_FRAME_INTERVAL, 1);

            mediaCodec.configure(format, null, null, MediaCodec.CONFIGURE_FLAG_ENCODE);
            inputSurface = mediaCodec.createInputSurface();
            mediaCodec.start();

            new Thread(() -> {
                MediaCodec.BufferInfo bufferInfo = new MediaCodec.BufferInfo();
                while (true) {
                    int outputBufferId = mediaCodec.dequeueOutputBuffer(bufferInfo, 10000);
                    if (outputBufferId >= 0) {
                        ByteBuffer outputBuffer = mediaCodec.getOutputBuffer(outputBufferId);
                        srsFlvMuxer.sendVideo(outputBuffer, bufferInfo);
                        mediaCodec.releaseOutputBuffer(outputBufferId, false);
                    }
                }
            }).start();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void setupVirtualDisplay() {
        virtualDisplay = mediaProjection.createVirtualDisplay("ScreenRecordingService",
                WIDTH, HEIGHT, DENSITY,
                DisplayManager.VIRTUAL_DISPLAY_FLAG_AUTO_MIRROR,
                inputSurface, null, null);
    }

    private void stopScreenRecording() {
        if (virtualDisplay != null) {
            virtualDisplay.release();
            virtualDisplay = null;
        }
        if (mediaProjection != null) {
            mediaProjection.unregisterCallback(mediaProjectionCallback);
            mediaProjection.stop();
            mediaProjection = null;
        }
        if (mediaCodec != null) {
            mediaCodec.stop();
            mediaCodec.release();
            mediaCodec = null;
        }
        stopSelf();
        srsFlvMuxer.stop();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        stopScreenRecording();
    }


    @Override
    public void onConnectionSuccessRtmp() {
        Log.d(TAG, "Connection success");
    }

    @Override
    public void onConnectionFailedRtmp(String reason) {
        Log.e(TAG, "Connection failed: " + reason);
        stopScreenRecording();
    }

    @Override
    public void onNewBitrateRtmp(long bitrate) {
        Log.d(TAG, "New bitrate: " + bitrate);
    }

    @Override
    public void onDisconnectRtmp() {
        Log.d(TAG, "Disconnected");
    }

    @Override
    public void onAuthErrorRtmp() {
        Log.e(TAG, "Auth error");
    }

    @Override
    public void onAuthSuccessRtmp() {
        Log.d(TAG, "Auth success");
    }
}
